/*

Ce fichier contient les fonctions grant l'affichage des masses et/ou les tiges
qui les relient, dans le cadre de mon projet de TIPE de 2007 (maths sp 1e anne.)

Auteur : Thibault Raffaillac <thibaultraffaillac@yahoo.fr>
Cr le : 09/03/07

*/

#ifndef TRAF_TIPE_AFFICHAGE_MASSES_H
#define TRAF_TIPE_AFFICHAGE_MASSES_H

#define DUREE_ENTRE_CHAQUE_IMAGE 40

typedef enum {
   CHOIX_CONTINUER,
   CHOIX_QUITTER,
   CHOIX_REINITIALISER,
   CHOIX_PRISE_IMAGE,
   CHOIX_PAUSE
} AffChoix;

typedef enum {
   N_AFFICHE_PAS_IMAGE,
   OK_INTERVALLE_A_AUGMENTER,
   OK_INTERVALLE_PARFAIT,
   OK_INTERVALLE_A_DIMINUER
} AffImage;

/*Cette fonction initialise la SDL et cre un cran par dfaut de taille voulue.
La fonction renvoie 0 aprs s'tre droule sans encombre et -1 en cas d'chec.*/
int initAffichage (int largeurEntree, int hauteurEntree, int nombreMassesEntre, int longueurFilEntree);

/*Cette fonction rinitialise des paramtres lis  l'affichage en vue de
relancer le fil.

Si la fonction est appele alors que l'affichage n'a pas encore t initialis,
alors elle renverra -1.*/
int reinitAffichage (void);

/*Cette fonction affiche le fil  partir des donnes des angles des tiges.

Si la fonction est appele alors que l'affichage n'a pas t initialis,
alors elle renverra -1.*/
int afficheFilAngles (double tableauMasses[]);

/*Cette fonction dtermine en fonction de l'intervalle de temps qui lui est
envoy si l'image qui lui correspond doit tre affiche, en respectant ces rgles:
_une image est affiche toutes les 40ms en moyenne, mais cette dure peut tre
change.
_si les intervalles de temps dpassent toujours 40ms, toutes les images seront
affiches.
_si le temps rel se situe aprs le temps thorique d'affichage d'une image,
alors le fil est color en rouge.
_dans l'autre cas o le calcul n'est donc pas en retard, on temporise jusqu' 10ms
avant l'affichage thorique de manire  toujours conserver une marge de temps.
_si une image doit tre affiche, la fonction indique en plus si l'intervalle
doit tre augment, s'il peut tre diminu ou s'il est parfait, en renvoyant
OK_INTERVALLE_A_AUGMENTER, OK_INTERVALLE_A_DIMINUER ou OK_INTERVALLE_PARFAIT.
Si l'image ne doit pas tre affiche, la fonction renvoie N_AFFICHE_PAS_IMAGE.
_la fonction reoit aussi une valeur ralenti qui dsigne un coefficient
multiplicateur de l'intervalle de temps, 1 signifiant alors "pas de ralenti".
Plus d'images seront bien sr dclares comme  afficher.

Si la fonction est appele alors que l'affichage n'a pas encore t initialis
ou si le ralenti est infrieur  1, alors elle renverra -1.

Attention ! intervalleTemps doit tre donne en millisecondes.*/
int verifieEtTemporise (double intervalleTemps, double ralenti);

/*Cette fonction colore le fil en rouge.

Si elle est appele alors que l'affichage n'a pas t initialis, alors elle
renverra -1.*/
int filEnRouge (void);

/*Cette fonction intercepte les vnements, les traite (pause, sauvegarde
d'image, rinitialisation de l'oscillateur) et renvoie CHOIX_CONTINUER pour
continuer, CHOIX_QUITTER pour quitter et CHOIX_REINITIALISER pour rinitialiser.

Si la fonction est appele alors que l'affichage n'a pas encore t initialis,
alors elle renverra -1.
*/
int gereEvenements (void);

/*Cette fonction impose une pause :) pendant laquelle on peut au choix prendre
une photo, ou rinitialiser. Elle renvoie CHOIX_QUITTER, CHOIX_CONTINUER ou
CHOIX_REINITIALISER.

Si la fonction est appele alors que l'affichage n'a pas encore t initialis,
alors elle renverra -1.
*/
int pause (void);

/*Cette fonction sauvegarde l'image actuellement affiche dans un fichier, en
prenant garde de ne pas en craser un prcdent au passage.
Elle renvoie 0 si l'opration s'est droule avec succs et -1 en cas d'chec.*/
int priseImage (void);

/*Cette fonction arrte la SDL.*/
void arretAffichage (void);

#endif
